"""
"""

class NodeAttribute(object):
    def __init__(self, value): self.value = value
class NodeClass(NodeAttribute): pass
class NodeId(NodeAttribute): pass
class NodeSingle(NodeAttribute): pass

class NodeNamed(NodeAttribute):
    def __init__(self, name, value):
        self.name = name
        self.value = value

class NodeBlock(list): pass
class NodeLine(list): pass

class NodeJinja(object):
    def __init__(self, name, args, block=None):
        self.name = name
        self.args = args
        self.block = block

class NodeTag(object):
    def __init__(self, name):
        self.name = name
        self.classes = set()
        self.id = None
        self.attribs = dict()
        self.singles = set()
        self.block = None
        self.line = None
        self.selfclosing = False

    def add_attribute(self, a):
        if isinstance(a, NodeId):
            self.id = a.value
        if isinstance(a, NodeClass):
            self.classes.add(a.value)
        if isinstance(a, NodeNamed):
            self.attribs[a.name] = a.value
        if isinstance(a, NodeSingle):
            self.singles.add(a.value)
        return self

    def set_attributes(self, attrs=[]):
        for a in attrs: self.add_attribute(a)
        return self

    def set_block(self, block):
        self.block = block
        return self

    def set_line(self, line):
        self.line = line
        return self

    def set_selfclosing(self, selfclosing=True):
        self.selfclosing = selfclosing
        return self

class NodeExtern(NodeTag):
    def __init__(self, name):
        self.extern_name = name

        if name in ["css", "sass", "scss", "stylus"]:
            super(NodeExtern, self).__init__("style")
            self.set_attributes([
                NodeNamed("rel", "stylesheet"),
                NodeNamed("type", "text/css")
            ])
        elif name in ["javascript", "coffeescript", "coco", "coffee"]:
            super(NodeExtern, self).__init__("script")
            self.set_attributes([
                NodeNamed("type", "text/javascript")
            ])
        else:
            super(NodeExtern, self).__init__(name)

    def set_block(self, block):
        self.block = block
        return self



#!/usr/bin/env python
from pwpeg import *
from pwpeg.helpers import *


##############################################################
# Start of included code
import re

from nodes import *

try:
    u = unicode
except Exception as e:
    u = str

def getindent(start):
    return len(start) - start.rfind('\n') - 1

_is = [] # _is is short for "Indent Stack"
def checkindent(indent, start):
    sindent = getindent(start)
    ci = len(_is) # Current indent index

    if indent == ci:
        # We're looking at a new indentation, to potentially push it
        # on the stack if we can.

        if ci == 0 or sindent > _is[-1]:
            _is.append(sindent)
            return True

        # It is not higher, so we just tell the parser that we
        # can't continue on that indentation level.
        return False

    if indent == ci - 1:
        # We're in the current block and want to check if this
        # line is correctly indented.

        # We're at the correct level, so we just return True
        if sindent >= _is[-1]: return True

        # We're looking at a deindent, so we pop the current indent
        # level from the stack.
        _is.pop()
        return False

def higherindent(start):
    sindent = len(start) - start.rfind('\n') - 1
    return sindent > _is[-1]


def lastindent():
    return _is[-1]

allbut_jinja_print = allbut_regexp("}}", '"}}"')

# End of included code
##############################################################

# Forward declaration of Regular rules
EOL = Rule().set_name("EOL")
_space = Rule().set_name("_space")
access_or_funcall = Rule().set_name("access_or_funcall")
attrib = Rule().set_name("attrib")
attribute = Rule().set_name("attribute")
blockraw = Rule().set_name("blockraw")
cls = Rule().set_name("cls")
comment = Rule().set_name("comment")
extern_tag = Rule().set_name("extern_tag")
fullspace = Rule().set_name("fullspace")
id = Rule().set_name("id")
ident = Rule().set_name("ident")
single_identifier = Rule().set_name("single_identifier")
single_prop = Rule().set_name("single_prop")
string = Rule().set_name("string")
tag = Rule().set_name("tag")
variable = Rule().set_name("variable")
variable_component = Rule().set_name("variable_component")
variable_dotted = Rule().set_name("variable_dotted")
vident = Rule().set_name("vident")


# Forward declaration of Function rules
block = FunctionRule().set_name("block")
blockstart = FunctionRule().set_name("blockstart")
delimited_line = FunctionRule().set_name("delimited_line")
line = FunctionRule().set_name("line")
lineelt = FunctionRule().set_name("lineelt")


# Function Rules implementation
def _block(indent):
    def fn_6(lines):
        
                if indent == 0:
                    global _is
                    _is = []
                

    def action_7(lines):
        return NodeBlock(lines)

    return Rule(
        OneOrMore(Rule(
            fullspace,
            lambda start: ( checkindent(indent, start) ) ,
            blockstart.instanciate(indent)
        ).set_action(lambda start, block: ([start, block]) )),
        fn_6
    ).set_action(action_7)
block.set_fn(_block)

def _blockstart(indent):
    def action_4(t, _1, a, _3, i, _5, b):
        return t.set_attributes(a).set_block(b).set_line(i)

    def action_5(c, _1, _2):
        return c

    return Either(
        Rule(
            tag,
            Optional(_space),
            ZeroOrMore(attribute),
            Optional(_space),
            Optional(line),
            EOL,
            block.instanciate(indent + 1)
        ).set_action(action_4),
        Rule(
            OneOrMore(attribute),
            Optional(line),
            EOL,
            Optional(block.instanciate(indent + 1))
        ).set_action(lambda a, i, _2, b: (NodeTag("div").set_attributes(a).set_block(b).set_line(i)) ),
        Rule(
            "%",
            Optional(_space),
            ident,
            re.compile('[^\n]*'),
            EOL,
            Optional(block.instanciate(indent + 1))
        ).set_action(lambda _0, _1, j, c, _4, b: (NodeJinja(j, c, b)) ),
        Rule(
            extern_tag,
            ZeroOrMore(attribute),
            Optional(_space),
            EOL,
            Optional(blockraw),
            Optional(EOL)
        ).set_action(lambda t, a, _2, _3, b, _5: (t.set_attributes(a).set_block(b)) ),
        Rule(
            line,
            Optional(_space),
            Optional(EOL)
        ).set_action(action_5),
        Rule(
            Optional(_space),
            Optional(EOL)
        ).set_action(lambda s, e: ((s or "")) )
    )
blockstart.set_fn(_blockstart)

def _delimited_line(delim):
    return Rule(
        delim,
        Optional(line.instanciate(delim)),
        delim
    ).set_action(lambda _0, l, _2: (l or "") )
delimited_line.set_fn(_delimited_line)

def _line(terminator=EOL):
    return Rule(
        Optional(_space),
        OneOrMore(lineelt.instanciate(terminator))
    ).set_action(lambda _0, e: (NodeLine(e)) )
line.set_fn(_line)

def _lineelt(terminator):
    return Either(
        Rule(
            "@/",
            ident,
            ZeroOrMore(attribute)
        ).set_action(lambda _0, i, a: (NodeTag(i).set_selfclosing().set_attributes(a)) ),
        Rule(
            tag,
            Optional(_space),
            ZeroOrMore(attribute),
            Optional(_space),
            "[",
            Optional(line.instanciate("]")),
            "]"
        ).set_action(lambda t, _1, a, _3, _4, i, _6: (t.set_attributes(a).set_line(i or "")) ),
        Rule(
            tag,
            Optional(_space),
            ZeroOrMore(attribute),
            Optional(_space),
            Optional(line.instanciate(terminator))
        ).set_action(lambda t, _1, a, _3, i: (t.set_attributes(a).set_line(i)) ),
        comment,
        variable,
        Rule(
            '\\',
            re.compile('.')
        ).set_action(lambda _0, t: (t) ),
        Rule(
            Not(terminator),
            re.compile('.')
        ).set_action(lambda c: (c) )
    )
lineelt.set_fn(_lineelt)



# Simple Rules implementation
EOL.set_productions("\n")

def action_0(c):
    return re.sub("#\s[^\n]*", "", c)

_space.set_productions(
    re.compile('[ \t]*(#\s+.*$)?')
).set_action(action_0)

access_or_funcall.set_productions(
    Either(
        Balanced.instanciate("[", "]", "\\"),
        Balanced.instanciate("(", ")", "\\")
    )
).set_action(lambda b: ("".join(b)) )

attrib.set_productions(
    ident,
    "=",
    Either(
        string,
        line.instanciate(re.compile('\s'))
    )
).set_action(lambda key, _1, value: (NodeNamed(key, value)) )

attribute.set_productions(
    Optional(_space),
    Either(
        id,
        cls,
        single_prop,
        attrib
    )
).set_action(lambda _0, a: (a) )

def action_2(start, contents):
    # We just nuke the indentation from the raw text.
    return [start, contents]

def action_3(lines):
    return [lastindent(), lines]

blockraw.set_productions(
    OneOrMore(Rule(
        fullspace,
        lambda start: ( higherindent(start) ) ,
        re.compile('[^\n]*')
    ).set_action(action_2))
).set_action(action_3)

cls.set_productions(
    ".",
    single_identifier
).set_action(lambda _0, i: (NodeClass(i)) )

comment.set_productions(
    re.compile('\s*#\s[^\n]*')
).set_action(lambda _0: ("") )

extern_tag.set_productions(
    ":",
    Either(
        "plain",
        "javascript",
        "css",
        "coco",
        "coffeescript",
        "coffee",
        "sass",
        "scss",
        "stylus"
    )
).set_action(lambda _0, i: (NodeExtern(i)) )

def action_1(c):
    return re.sub("#\s[^\n]*", "", c)

fullspace.set_productions(
    re.compile('([ \t]*(#\s+.*)?(\n|$))*[ \t]*')
).set_action(action_1)

id.set_productions(
    '#',
    single_identifier
).set_action(lambda _0, i: (NodeId(i)) )

ident.set_productions(re.compile('[-$\w:]+', re.U))

single_identifier.set_productions(
    OneOrMore(Either(
        "\\$",
        variable,
        re.compile('[^\s#\.]')
    ))
).set_action(lambda i: ("".join(i)) )

single_prop.set_productions(
    '\\',
    single_identifier
).set_action(lambda _0, i: (NodeSingle(i)) )

string.set_productions(Either(
    delimited_line.instanciate("\""),
    delimited_line.instanciate("'")
))

tag.set_productions(
    "@",
    Not("/"),
    ident
).set_action(lambda _0, i: (NodeTag(i)) )

variable.set_productions(Either(
    Rule(
        "$",
        OneOrMoreSeparated.instanciate(variable_dotted, "|")
    ).set_action(lambda _0, i: (u("{{{{ {0} }}}}").format(u("|").join(i))) ),
    Rule(
        "{{",
        allbut_jinja_print,
        "}}"
    ).set_action(lambda b, c, e: (b + c + e) )
))

variable_component.set_productions(
    vident,
    ZeroOrMore(access_or_funcall)
).set_action(lambda i, a: (i + "".join(a)) )

variable_dotted.set_productions(
    OneOrMoreSeparated.instanciate(variable_component, ".")
).set_action(lambda i: (u(".").join(i)) )

vident.set_productions(re.compile('[a-zA-Z_][a-zA-Z0-9_]*'))





import os.path
import os
import getpass

from jinja2 import  Environment, TemplateSyntaxError
from jinja2.ext import Extension

from pwpeg import Parser

from . import niml
from . import printer

class NimlExtension(Extension):

    def __init__(self, environment, cache=True):
        super(NimlExtension, self).__init__(environment)

        environment.extend(
            niml_compact=False,
            niml_file_extensions=('.niml',),
        )

        self.parser = Parser(niml.block.instanciate(0))

        self.cache = cache
        if cache:
            try:
                dirname = '/tmp/{0}/niml-ext'.format(getpass.getuser())
                self.dirname = dirname
                os.makedirs(dirname)
            except Exception as e:
                if e.errno != 17: # file exists
                    self.dirname = None


    def preprocess(self, source, name, filename=None):
        if name is None or os.path.splitext(name)[1] not in \
            self.environment.niml_file_extensions:
            return source

        if self.dirname:
            try:
                f = file(os.path.join(self.dirname, '{0}{1}'.format(os.path.basename(name), hash(source))), 'r')
                str = f.read().decode('utf-8')
                f.close()
                return str
            except Exception as e:
                pass

        visitor = printer.NodeVisitor(self.environment.niml_compact)

        try:
            root = self.parser.parse(source)
            result = visitor.visit(root)

            if self.dirname:
                try:
                    f = file(os.path.join(self.dirname, '{0}{1}'.format(os.path.basename(name), hash(source))), 'w')
                    f.write(result.encode('utf-8'))
                    f.close()
                except Exception as e:
                    pass

            return result
        except TemplateSyntaxError as e:
            print(result)
            raise TemplateSyntaxError(e.message, e.lineno, name=name, filename=filename)



import re
from subprocess import Popen, PIPE

from .nodes import NodeJinja, NodeBlock, NodeLine
from .niml import getindent

re_endspc = re.compile("([ \t]*\n)*$")
def addend(txt, end):
    return re_endspc.sub(lambda m: end + m.group(0), txt)

def adjuststarts(indent, linelists):
    lowest = None

    for l in linelists:
        i = getindent(l[0])
        if lowest is None or i < lowest:
            lowest = i

    if indent is None:
        result = [[l[0][:-lowest], l[1]] for l in linelists]
        result[0][0] = "\n" + result[0][0]
        return result
    else:
        finalindent = lowest - indent

        result = [[l[0][:-finalindent], l[1]] for l in linelists]
        # First line has a strange indent.
        result[0][0] = "\n" + result[0][0]# [:-indent]
        return result

def run(cmd, stdin=None):
    p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
    res = p.communicate(stdin) if stdin else p.communicate()
    if res[1]:
        raise Exception(res[1])
    return res[0]

def compile_coco(txt):
    return run(["coco", "-bpce", txt])

def compile_coffee(txt):
    return run(["coffee", "-bce", txt])

def compile_sass(txt):
    return run(["sass", "-s"], txt)

def compile_scss(txt):
    return run(["sass", "-s", "--scss"], txt)

def compile_stylus(txt):
    return run(["stylus"], txt)

# Python 3 compatibility gimmick to avoid unicode errors.
try:
    unicode = unicode
except:
    unicode = str

class NodeVisitor(object):
    def __init__(self, compact=False):
        self.compact = compact
        if compact:
            self.joiner = unicode(" ")
        else:
            self.joiner = unicode("\n")

    def visit(self, node):
        return getattr(self, node.__class__.__name__)(node)

    def str(self, s):
        return s.decode("utf-8")

    def unicode(self, u):
        return u

    def NoneType(self, value):
        return ""

    def NodeLine(self, line):
        res = map(lambda e: self.visit(e), line)
        return unicode("").join(res)

    def NodeTag(self, tag):
        op = []
        if tag.classes:
            op.append(unicode('class="{0}"').format(" ".join([t for t in tag.classes ])))
        if tag.id:
            op.append(unicode('id="{0}"').format(tag.id))
        if tag.attribs:
            op.append(unicode(" ").join([ unicode('{0}="{1}"').format(key, self.visit(value)) for key, value in tag.attribs.items()] ))
        if tag.singles:
            op.append(unicode(" ").join(tag.singles))
        op = unicode(" ").join(op)
        if op: op = unicode(" ") + op

        selfclosing = "/" if tag.selfclosing else ""

        res = []
        res.append(unicode("<{0}{1}{2}>").format(tag.name, op, selfclosing) + self.visit(tag.line))

        if not selfclosing:
            if tag.block:
                res.append(addend(self.visit(tag.block), unicode("</{0}>").format(tag.name)))
            else:
                res[0] = addend(res[0], unicode("</{0}>").format(tag.name))

        return (self.joiner if not self.compact else unicode("")).join(res)

    def NodeBlock(self, block):
        res = []

        for i, b in enumerate(block):
            res.append( unicode("{0}{1}").format("" if self.compact else b[0], self.visit(b[1])) )

            if isinstance(b[1], NodeJinja):
                nxt = None
                if i < len(block) - 1:
                    nxt = block[i + 1][1]

                if b[1].name in ["if", "elif", "else"]:
                    if not isinstance(nxt, NodeJinja) or nxt.name not in ["elif", "else"]:
                        res[-1] = addend(res[-1], "{{% endif %}}".format(b[1].name))

                if b[1].name in ["for", "macro", "filter", "block"]:
                    res[-1] = addend(res[-1], unicode("{{% end{0} %}}").format(b[1].name))

            #if b[2]:
            #    for l in b[2].split("\n")[:-1]:
            #        res.append("")
        return self.joiner.join(res)

    def NodeExtern(self, node):
        if not node.block:
            return ""

        res = []
        indent, lines = node.block

        if node.name == "plain":
            lines = adjuststarts(indent, lines)
            txt = "".join([b[0] + b[1] for b in lines])
            res.append(txt)

        # We're going to format code, so we eat all the useless indent.
        else:
            lines = adjuststarts(None, lines)
            txt = "".join([b[0] + b[1] for b in lines])

            if node.extern_name == "coco":
                txt = compile_coco(txt)
            if node.extern_name in ["coffeescript", "coffee"]:
                txt = compile_coffee(txt)
            if node.extern_name == "scss":
                txt = compile_scss(txt)
            if node.extern_name == "sass":
                txt = compile_sass(txt)
            if node.extern_name == "stylus":
                txt = compile_stylus(txt)

            node.block = None
            node.set_line(NodeLine([ txt ]))
            res.append(self.NodeTag(node))

        return self.joiner.join(res)

    def NodeJinja(self, j):
        return unicode("{{% {0} {1} %}}").format(j.name, j.args.strip()) + ("" if not j.block else ("\n" if not self.compact else "") + self.visit(j.block))




#!/usr/bin/env python

from distutils.core import setup

setup(name='niml',
      version='0.12.6',
      description='Python niml parser',
      author='Christophe Eymard',
      author_email='christophe.eymard@ravelsoft.com',
      url='http://niml.github.com/',
      packages=['niml'],
      scripts=['scripts/niml'],
      requires=['pwpeg']
     )
